#include <nds.h>
#include <nds/arm9/console.h>
#include "pacman_bin.h"

#include "console.h"
#include "sound.h"
#include "mediasrc.h"

bool theRedrawEntireFrameIndicator = true;

Console* theConsole = (Console*) NULL;
Sound* theSound = (Sound*) NULL;

uInt8 myPaletteR[256], myPaletteG[256], myPaletteB[256];

void setuppalette();
void updateframebuffer();

int main(void)
{
	powerON(POWER_ALL);

	videoSetMode(MODE_FB0);
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);
	vramSetBankA(VRAM_A_LCD);
	vramSetBankC(VRAM_C_SUB_BG); 

	SUB_BG0_CR = BG_MAP_BASE(31);
	BG_PALETTE_SUB[255] = RGB15(31,31,31);
	
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);

    theSound = new Sound();
	theConsole = new Console(pacman_bin, pacman_bin_size, "pacman.bin", *theSound);
	consolePrintf("Console created.\n");
	
	for(int i = 0; i < SCREEN_WIDTH * SCREEN_HEIGHT; ++i) 
		VRAM_A[i] = RGB15(0,0,31);
		
	setuppalette();

	while(1)
	{
		theConsole->update();
		updateframebuffer();
	}

	return 0;
}

void updateframebuffer()
{
	uInt8* currentFrame  = theConsole->myMediaSource->currentFrameBuffer();
	uInt32 width         = theConsole->myMediaSource->width();
	uInt32 height        = theConsole->myMediaSource->height();

	for(uInt32 y = 0; y < width * height; ++y)
	{
	    uInt8 v = currentFrame[y];
		VRAM_A[y] = RGB15(myPaletteR[v],myPaletteG[v],myPaletteB[v]);
	}
}

void setuppalette()
{
	const uInt32* gamePalette = theConsole->myMediaSource->palette();

	for(uInt32 i = 0; i < 256; ++i)
	{
		uInt8 r, g, b;

		r = (uInt8) ((gamePalette[i] & 0x00ff0000) >> 16);
		g = (uInt8) ((gamePalette[i] & 0x0000ff00) >> 8);
		b = (uInt8) (gamePalette[i] & 0x000000ff);

		myPaletteR[i] = r;
		myPaletteG[i] = g;
		myPaletteB[i] = b;
	}
	
	for(int i = 0; i < 256; ++i) 
		VRAM_A[i] = RGB15(myPaletteR[i],myPaletteG[i],myPaletteB[i]);
}
